from glob import glob
import os


class NbaSeason:
    @staticmethod
    def my_import(name, classname):
        components = name.split('.')
        mod = __import__(name)
        for comp in components[1:]:
            mod = getattr(mod, comp)

        requested_class = getattr(mod, classname)
        return requested_class

    def __init__(self):
        """
        Load all Season Objects
        """
        self.seasons = {}
        class_dir = os.path.join(os.path.dirname(os.path.abspath(__file__)))
        class_dir += '/seasons/season_*.py'

        for file in glob(class_dir):
            filename = os.path.splitext(os.path.basename(file))[0]
            name, date = filename.split('_')

            season_file = 'sports.source.nba.seasons.{0}'.format(filename)
            my_classname = "NBA_Season_{0}".format(date)
            my_class = self.my_import(season_file, my_classname)
            self.seasons[date]  = my_class()

    def get(self, season:int):
        """

        Args:
            season: Integer for the start of the season

        Returns:
            NBA_Season_YYYY class

        """
        if season not in self.seasons.keys():
            raise KeyError("{0} does not exist or has not been modeled".format(season))

        return self.seasons[season]


from sports.nba.nba_team import NBA_Team


class BrooklynNets(NBA_Team):
    """
    NBA Golden State Warriors Static Information

    """
    full_name   = "Brooklyn Nets"
    name        = "Nets"
    team_id     = 1610612751

    def __init__(self):
        """
        """
        super().__init__()



from sports.nba.nba_team import NBA_Team

class AtlantaHawks(NBA_Team):
    """
    NBA Atlanta Hawks Static Information

    """
    full_name = "Atlanta Hawks"
    name = "Hawks"
    team_id = 1610612737

    def __init__(self):
        """
        """
        super().__init__()


from sports.nba.nba_team import NBA_Team


class ChicagoBulls(NBA_Team):
    """
    NBA Atlanta Hawks Static Information

    """
    full_name = "Chicago Bulls"
    name = "Bulls"
    team_id = 1610612741

    def __init__(self):
        """
        """
        super().__init__()



import xml.sax
from sports.models import Game
from sports.models import Inning
from sports.models.AtBat import AtBat

class InningsAllXml(xml.sax.ContentHandler):
    def __init__(self):
        self.currentInning = None
        self.currentAtBat  = None

        self.game = Game()

    def startElement(self, name, attrs):
        if name == 'game':
            # Add all the attrbutes for the Game
            for k, v in attrs.items():
                setattr(self.game, k, v)
        elif name == 'inning':
            # Finished Processing the Previous Inning
            if self.currentInning != None:
              self.game.innings.append(self.currentInning)

            # Add all the attributes for the Inning
            self.currentInning = Inning()
            for k,v in attrs.items():
                setattr(self.currentInning,k,v)
        elif name == 'atbat':
            # Finished processing the previous at bat
            if self.currentAtBat != None:
                self.currentInning.at_bats.append(self.currentAtBat)
            self.currentAtBat = AtBat()
            for k,v in attrs.items():
                setattr(self.currentAtBat,k,v)
        elif name == 'pitch':
            self.currentAtBat.pitches.append(attrs.items())
        elif name == 'runner':
            self.currentAtBat.runners.append(dict(attrs))


class NbaTeams:
    """
    A list of all NBA teams and their stats in a given year.

    Finds and retrieves a list of all NBA teams from
    www.basketball-reference.com and creates a Team instance for every team
    that participated in the league in a given year. The Team class comprises
    a list of all major stats and a few identifiers for the requested season.

    Parameters
    ----------
    year : string (optional)
        The requested year to pull stats from.
    season_file : string (optional)
        Optionally specify the filename of a local file to use to pull data
        instead of downloading from sports-reference.com. This file should be
        of the Season page for the designated year.
    """
    def __init__(self, year=None, season_file=None):
        self._teams = []

        # team_data_dict, year = _retrieve_all_teams(year, season_file)
        # self._instantiate_teams(team_data_dict, year)

    def __getitem__(self, abbreviation):
        """
        Return a specified team.

        Returns a team's instance in the Teams class as specified by the team's
        abbreviation.

        Parameters
        ----------
        abbreviation : string
            An NBA team's three letter abbreviation (ie. 'DET' for Detroit
            Pistons).

        Returns
        -------
        Team instance
            If the requested team can be found, its Team instance is returned.

        Raises
        ------
        ValueError
            If the requested team is not present within the Teams list.
        """
        for team in self._teams:
            if team.abbreviation.upper() == abbreviation.upper():
                return team
        raise ValueError('Team abbreviation %s not found' % abbreviation)

    def __call__(self, abbreviation):
        """
        Return a specified team.

        Returns a team's instance in the Teams class as specified by the team's
        abbreviation. This method is a wrapper for __getitem__.

        Parameters
        ----------
        abbreviation : string
            An NBA team's three letter abbreviation (ie. 'DET' for Detroit
            Pistons).

        Returns
        -------
        Team instance
            If the requested team can be found, its Team instance is returned.
        """
        return self.__getitem__(abbreviation)

    def __str__(self):
        """
        Return the string representation of the class.
        """
        teams = [f'{team.name} ({team.abbreviation})'.strip()
                 for team in self._teams]
        return '\n'.join(teams)

    def __repr__(self):
        """
        Return the string representation of the class.
        """
        return self.__str__()

    def __iter__(self):
        """Returns an iterator of all of the NBA teams for a given season."""
        return iter(self._teams)

    def __len__(self):
        """Returns the number of NBA teams for a given season."""
        return len(self._teams)

    def _instantiate_teams(self, team_data_dict, year):
        """
        Create a Team instance for all teams.

        Once all team information has been pulled from the various webpages,
        create a Team instance for each team and append it to a larger list of
        team instances for later use.

        Parameters
        ----------
        team_data_dict : dictionary
            A ``dictionary`` containing all stats information in HTML format as
            well as team rankings, indexed by team abbreviation.
        year : string
            A ``string`` of the requested year to pull stats from.
        """
        if not team_data_dict:
            return
        for team_data in team_data_dict.values():
            team = Team(team_data=team_data['data'],
                        rank=team_data['rank'],
                        year=year)
            self._teams.append(team)

    @property
    def dataframes(self):
        """
        Returns a pandas DataFrame where each row is a representation of the
        Team class. Rows are indexed by the team abbreviation.
        """
        frames = []
        for team in self.__iter__():
            frames.append(team.dataframe)
        return pd.concat(frames)


class NbaBoxScore(object):
    def _set_attributes(self, attributes):
        for key, value in attributes.items():
            setattr(self, key, value)

    def __init__(self):
        self.players = []
        self.teams   = []

        # Set via the boxscore_summary endpoint
        self.attendance         =   None
        self.game_date          =   None
        self.game_time          =   None
        self.home_team_id       =   None
        self.visitor_team_id    =   None
        self.winning_team_id    =   None    #Not sure where set yet



from sports.nba.nba_team import NBA_Team


class NewYorkKnickerbockers(NBA_Team):
    """
    NBA's New York Knickerbockers Static Information

    """

    full_name = "New York Knickerbockers"
    name = "Knickerbockers"
    team_id = 1610612752

    def __init__(self):
        """
        """
        super().__init__()



from ..nba.constants import *
import types
from enum import Enum
from inspect import isclass

try:
    import importlib.resources as pkg_resources
except ImportError:
    # Try backported to PY<37 `importlib_resources`.
    import importlib_resources as pkg_resources

import requests
import logging
import json
from pandas import DataFrame

################################
# Decorators for the NBA Class #
################################
def clean_inputs(func):
    """
    Iterates over a function's parameters checking for Enum, if one is found, the value is used instead

    Args:
        func: Underlying function to be wrapped

    Returns: Wrapped function

    """

    def new_func(*args, **kwargs):
        cleaned_args = []
        for i in range(len(args)):
            if isclass(type(args[i])) and issubclass(type(args[i]), Enum):
                clean_arg = args[i].value
                cleaned_args.append(clean_arg)
            else:
                cleaned_args.append(args[i])

        for key, val in kwargs.items():
            if isclass(type(val)) and issubclass(type(val), Enum):
                kwargs[key] = val.value

        data = func(*cleaned_args, **kwargs)
        return data

    return new_func


class StatsNbaApi:
    def __init__(self):
        # Get a logger
        logging.basicConfig(level=logging.INFO, format='%(asctime)s - %(levelname)s: %(message)s')
        self.logger = logging.getLogger('sportsdata')

        # Read the specification file
        self.specs = json.loads(pkg_resources.read_text('sportsdata.data', 'stats.nba.com.json'))
        self.logger.info(self.specs.keys())

        self.parameters = {}
        for param in self.specs['parameters']:
            self.parameters[param['name']] = {
                'default': param['default'],
                'values': param['values']
            }

        # Create methods for each api endpoint
        for endpoint in self.specs['stats_endpoints']:
            self.add_api_method(endpoint)

    @staticmethod
    def _get_row_set(rs):
        data = []
        for row in rs['rowSet']:
            data_point = dict(zip([h.lower() for h in rs['headers']], row))
            data.append(data_point)
        return data

    @staticmethod
    def _get_data_frames(response, rename_to={}):
        """
        Parse the response for any results and load them into data frames
        Args:
            response:
            rename_to:

        Returns:
            All Result Sets Found as Data Frames

        """
        frames = {}
        info = json.loads(response.text)
        result_sets = info['resultSets']
        for rs in result_sets:
            rs_name = rs['name']
            if rs_name in rename_to.keys():
                rs_name = rename_to[rs_name]

            frames[rs_name] = DataFrame(rs['rowSet'], columns=rs['headers'])

        # Check if there is only one result, if so no need for a dictionary
        if len(frames) == 1:
            key = next(iter(frames))
            frames = frames[key]

        return frames

    @staticmethod
    def _get_dictionary(response):
        return json.loads(response.text)

    def add_api_method(self, endpoint):
        """
        Dynamically builds a method for each endpoint in the specification file
        :param endpoint:
        :param cls:
        :param name:
        :return:
        """

        def dynamic_method(self2: StatsNbaApi, **kwargs):
            url = endpoint['url']
            parameters = endpoint['parameters']
            self2.logger.info(url)
            self2.logger.info(kwargs)

            # Determine the ResponseType
            return_type = ReturnType.DICTIONARY.value
            if 'ReturnType' in kwargs:
                return_type = kwargs['ReturnType']

            url_parameters = {}
            # Add each parameter to the url
            for param in parameters:
                value = ''

                # Was this parameter passed into to the method?
                if param in kwargs:
                    value = kwargs[param]

                # Does this parameter have a default value
                elif param in self.parameters:
                    value = self.parameters[param]['default']

                # Is the value for the parameter a legal value for this parameter
                # if param in self.parameters and value not in self.parameters[param]['values']:
                #     self.logger.warning(f"The value '{value}' is not a legal value for '{param}'")
                #     value = ''

                # Add the parameter and its value to the dictionary of url parameters
                url_parameters[param] = value

            self2.logger.info(url_parameters)
            request_headers = {
                'Host': 'stats.nba.com',
                'User-Agent': 'Mozilla/5.0 (Windows NT 10.0; Win64; x64; rv:72.0) Gecko/20100101 Firefox/72.0',
                'Accept': 'application/json, text/plain, */*',
                'Accept-Language': 'en-US,en;q=0.5',
                'Accept-Encoding': 'gzip, deflate, br',
                'x-nba-stats-origin': 'stats',
                'x-nba-stats-token': 'true',
                'Connection': 'keep-alive',
                'Referer': 'https://stats.nba.com/',
                'Pragma': 'no-cache',
                'Cache-Control': 'no-cache',
            }
            response = requests.get(url, params=url_parameters, headers=request_headers, timeout=10)

            if return_type == ReturnType.DICTIONARY or return_type == ReturnType.DICTIONARY.value:
                return_value = self2._get_dictionary(response)
            elif return_type == ReturnType.RESPONSE or return_type == ReturnType.RESPONSE.value:
                return_value = response
            elif return_type == ReturnType.DATA_FRAMES or return_type == ReturnType.DATA_FRAMES.value:
                return_value = self2._get_data_frames(response)

            # Close the response before leaving
            response.close()

            return return_value

        dynamic_method.__name__ = endpoint['name']
        setattr(self, dynamic_method.__name__, types.MethodType(dynamic_method, self))
